@@ -23,7 +23,7 @@ use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 use Image::ExifTool::Exif;
 
-$VERSION = '1.31';
+$VERSION = '1.35';
 
 sub ProcessKodakIFD($$$);
 sub ProcessKodakText($$$);
@@ -750,16 +750,18 @@ sub WriteKodakIFD($$$);
         Notes => 'Kodak only',
     },
     0xa8 => {
-        Name => 'UnknownNumber', # (was SerialNumber, but not unique for all cameras. ie C1013)
+        Name => 'UnknownNumber', # (was SerialNumber, but not unique for all cameras. eg. C1013)
         Condition => '$$self{Make} =~ /Kodak/i and $$valPt =~ /^([A-Z0-9]{1,11}\0|[A-Z0-9]{12})/i',
         Format => 'string[12]',
         Notes => 'Kodak only',
+        Writable => 0,
     },
     0xc4 => {
         Name => 'UnknownNumber', # (confirmed NOT to be serial number for Easyshare Mini - PH)
         Condition => '$$self{Make} =~ /Kodak/i and $$valPt =~ /^([A-Z0-9]{1,11}\0|[A-Z0-9]{12})/i',
         Format => 'string[12]',
         Notes => 'Kodak only',
+        Writable => 0,
     },
 );
 
@@ -843,6 +845,31 @@ sub WriteKodakIFD($$$);
     # 0x3fe undef[2540]
 );
 
+# Kodak PixPro S-1 maker notes (ref PH)
+# (similar to Ricoh::Type2 and GE::Main)
+%Image::ExifTool::Kodak::Type11 = (
+    # (can't currently write these)
+    GROUPS => { 0 => 'MakerNotes', 2 => 'Camera' },
+    NOTES =>q{
+        These tags are found in models such as the PixPro S-1.  They are not
+        writable because the inconsistency of Kodak maker notes is beginning to get
+        on my nerves.
+    },
+    0x0203 => {
+        Name => 'PictureEffect',
+        PrintConv => {
+            0 => 'None',
+            3 => 'Monochrome',
+            9 => 'Kodachrome',
+        },
+    },
+    # 0x0204 - ExposureComp or FlashExposureComp maybe?
+    0x0207 => 'KodakModel',
+    0x0300 => 'KodakMake',
+    0x0308 => 'LensSerialNumber',
+    0x0309 => 'LensModel',
+);
+
 # Kodak SubIFD0 tags (ref PH)
 %Image::ExifTool::Kodak::SubIFD0 = (
     WRITE_PROC => \&Image::ExifTool::Exif::WriteExif,
@@ -852,7 +879,7 @@ sub WriteKodakIFD($$$);
     0xfa02 => {
         Name => 'SceneMode',
         Writable => 'int16u',
-        Notes => 'may not be valid for some models', # ie. M580?
+        Notes => 'may not be valid for some models', # eg. M580?
         PrintConvColumns => 2,
         PrintConv => {
             1 => 'Sport',
@@ -874,8 +901,8 @@ sub WriteKodakIFD($$$);
             25 => 'Back Light',
             28 => 'Candlelight',
             29 => 'Sunset',
-            31 => 'Panorama Left-Right',
-            32 => 'Panorama Right-Left',
+            31 => 'Panorama Left-right',
+            32 => 'Panorama Right-left',
             33 => 'Smart Scene',
             34 => 'High ISO',
         },
@@ -1877,7 +1904,7 @@ sub CalculateRGBLevels(@)
 # Returns: 1 on success
 sub ProcessKodakText($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dataPt = $$dirInfo{DataPt};
     my $dirStart = $$dirInfo{DirStart} || 0;
     my $dirLen = $$dirInfo{DirLen} || length($$dataPt) - $dirStart;
@@ -1885,12 +1912,12 @@ sub ProcessKodakText($$$)
     $data =~ s/\0.*//s;     # truncate at null if it exists
     my @lines = split /[\n\r]+/, $data;
     my ($line, $success, @other, $tagInfo);
-    $exifTool->VerboseDir('Kodak Text');
+    $et->VerboseDir('Kodak Text');
     foreach $line (@lines) {
         if ($line =~ /(.*?):\s*(.*)/) {
             my ($tag, $val) = ($1, $2);
             if ($$tagTablePtr{$tag}) {
-                $tagInfo = $exifTool->GetTagInfo($tagTablePtr, $tag);
+                $tagInfo = $et->GetTagInfo($tagTablePtr, $tag);
             } else {
                 my $tagName = $tag;
                 $tagName =~ s/([A-Z])\s+([A-Za-z])/${1}_\U$2/g;
@@ -1901,7 +1928,7 @@ sub ProcessKodakText($$$)
                 $tagInfo = { Name => $tagName };
                 AddTagToTable($tagTablePtr, $tag, $tagInfo);
             }
-            $exifTool->HandleTag($tagTablePtr, $tag, $val, TagInfo => $tagInfo);
+            $et->HandleTag($tagTablePtr, $tag, $val, TagInfo => $tagInfo);
             $success = 1;
         } else {
             # strip off leading/trailing white space and ignore blank lines
@@ -1910,11 +1937,11 @@ sub ProcessKodakText($$$)
     }
     if ($success) {
         if (@other) {
-            $tagInfo = $exifTool->GetTagInfo($tagTablePtr, '_other_info');
-            $exifTool->FoundTag($tagInfo, \@other);
+            $tagInfo = $et->GetTagInfo($tagTablePtr, '_other_info');
+            $et->FoundTag($tagInfo, \@other);
         }
     } else {
-        $exifTool->Warn("Can't parse Kodak TextualInfo data", 1);
+        $et->Warn("Can't parse Kodak TextualInfo data", 1);
     }
     return $success;
 }
@@ -1925,21 +1952,21 @@ sub ProcessKodakText($$$)
 # Returns: 1 on success, otherwise returns 0 and sets a Warning
 sub ProcessKodakIFD($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dirStart = $$dirInfo{DirStart} || 0;
     return 1 if $dirStart <= 0 or $dirStart + 2 > $$dirInfo{DataLen};
     my $byteOrder = substr(${$$dirInfo{DataPt}}, $dirStart, 2);
     unless (Image::ExifTool::SetByteOrder($byteOrder)) {
-        $exifTool->Warn("Invalid Kodak $$dirInfo{Name} directory");
+        $et->Warn("Invalid Kodak $$dirInfo{Name} directory");
         return 1;
     }
     $$dirInfo{DirStart} += 2;   # skip byte order mark
     $$dirInfo{DirLen} -= 2;
-    if ($exifTool->{HTML_DUMP}) {
+    if ($$et{HTML_DUMP}) {
         my $base = $$dirInfo{Base} + $$dirInfo{DataPos};
-        $exifTool->HDump($dirStart+$base, 2, "Byte Order Mark");
+        $et->HDump($dirStart+$base, 2, "Byte Order Mark");
     }
-    return Image::ExifTool::Exif::ProcessExif($exifTool, $dirInfo, $tagTablePtr);
+    return Image::ExifTool::Exif::ProcessExif($et, $dirInfo, $tagTablePtr);
 }
 
 #------------------------------------------------------------------------------
@@ -1948,14 +1975,14 @@ sub ProcessKodakIFD($$$)
 # Returns: Exif data block (may be empty if no Exif data) or undef on error
 sub WriteKodakIFD($$$)
 {
-    my ($exifTool, $dirInfo, $tagTablePtr) = @_;
+    my ($et, $dirInfo, $tagTablePtr) = @_;
     my $dirStart = $$dirInfo{DirStart} || 0;
     return '' if $dirStart <= 0 or $dirStart + 2 > $$dirInfo{DataLen};
     my $byteOrder = substr(${$$dirInfo{DataPt}}, $dirStart, 2);
     return '' unless Image::ExifTool::SetByteOrder($byteOrder);
     $$dirInfo{DirStart} += 2;   # skip byte order mark
     $$dirInfo{DirLen} -= 2;
-    my $buff = Image::ExifTool::Exif::WriteExif($exifTool, $dirInfo, $tagTablePtr);
+    my $buff = Image::ExifTool::Exif::WriteExif($et, $dirInfo, $tagTablePtr);
     return $buff unless defined $buff and length $buff;
     # apply one-time fixup for length of byte order mark
     if ($$dirInfo{Fixup}) {
@@ -1985,7 +2012,7 @@ interpret Kodak maker notes EXIF meta information.
 
 =head1 AUTHOR
 
-Copyright 2003-2013, Phil Harvey (phil at owl.phy.queensu.ca)
+Copyright 2003-2014, Phil Harvey (phil at owl.phy.queensu.ca)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
